@@ -1,8 +1,9 @@
 #pragma once
 
-#include "torch/csrc/jit/ir.h"
+#include <torch/csrc/jit/ir.h>
 
-namespace torch { namespace jit {
+namespace torch {
+namespace jit {
 
 struct HashNode {
   size_t operator()(const Node* k) const;
@@ -12,4 +13,5 @@ struct EqualNode {
   bool operator()(const Node* lhs, const Node* rhs) const;
 };
 
-}}
+} // namespace jit
+} // namespace torch
