@@ -18,8 +18,10 @@ from caffe2.python import (
 )
 from caffe2.python.layers.layers import (
     AccessedFeatures,
+    almost_equal_schemas,
     get_key,
     IdList,
+    IdScoreList,
     InstantiationContext,
     is_request_only_scalar,
     set_request_only,
@@ -302,7 +304,6 @@ class TestLayers(LayersTestCase):
         self.assertEquals(embedding_after_training.all(), embedding_after_init.all())
 
 
-
     def testSparseLookupSumPooling(self):
         record = schema.NewRecord(self.model.net, schema.Struct(
             ('sparse', schema.Struct(
@@ -842,6 +843,24 @@ class TestLayers(LayersTestCase):
         self.run_train_net_forward_only()
         self.assertEqual(schema.Scalar((np.float32, tuple())), loss)
 
+    def testBPRLoss(self):
+        input_record = self.new_record(schema.Struct(
+            ('pos_prediction', schema.Scalar((np.float32, (1,)))),
+            ('neg_prediction', schema.List(np.float32)),
+        ))
+        pos_items = np.array([0.8, 0.9], dtype=np.float32)
+        neg_lengths = np.array([1, 2], dtype=np.int32)
+        neg_items = np.array([0.1, 0.2, 0.3], dtype=np.float32)
+        schema.FeedRecord(
+            input_record,
+            [pos_items, neg_lengths, neg_items]
+        )
+        loss = self.model.BPRLoss(input_record)
+        self.run_train_net_forward_only()
+        self.assertEqual(schema.Scalar((np.float32, tuple())), loss)
+        result = workspace.FetchBlob('bpr_loss/output')
+        np.testing.assert_array_almost_equal(np.array(1.24386, dtype=np.float32), result)
+
     def testBatchMSELoss(self):
         input_record = self.new_record(schema.Struct(
             ('label', schema.Scalar((np.float64, (1,)))),
@@ -2229,12 +2248,12 @@ class TestLayers(LayersTestCase):
         self.model.FeatureSparseToDense(input_record, input_specs)
 
         expected_accessed_features = {
-            float_features_column: AccessedFeatures(
-                float_features_type, set(float_features_ids)),
-            id_list_features_column: AccessedFeatures(
-                id_list_features_type, set(id_list_features_ids)),
-            id_score_list_features_column: AccessedFeatures(
-                id_score_list_features_type, set(id_score_list_features_ids)),
+            float_features_column: [
+                AccessedFeatures(float_features_type, set(float_features_ids))],
+            id_list_features_column: [
+                AccessedFeatures(id_list_features_type, set(id_list_features_ids))],
+            id_score_list_features_column: [
+                AccessedFeatures(id_score_list_features_type, set(id_score_list_features_ids))],
         }
 
         self.assertEqual(len(self.model.layers), 1)
@@ -2242,3 +2261,55 @@ class TestLayers(LayersTestCase):
             self.model.layers[0].get_accessed_features(),
             expected_accessed_features
         )
+
+    def test_get_key(self):
+        def _is_id_list(input_record):
+            return almost_equal_schemas(input_record, IdList)
+
+
+        def _is_id_score_list(input_record):
+            return almost_equal_schemas(input_record,
+                                        IdScoreList,
+                                        check_field_types=False)
+
+        def old_get_sparse_key_logic(input_record):
+            if _is_id_list(input_record):
+                sparse_key = input_record.items()
+            elif _is_id_score_list(input_record):
+                sparse_key = input_record.keys()
+            else:
+                raise NotImplementedError()
+            return sparse_key
+
+        id_score_list_record = schema.NewRecord(
+            self.model.net,
+            schema.Map(
+                schema.Scalar(
+                    np.int64,
+                    metadata=schema.Metadata(
+                        categorical_limit=1000
+                    ),
+                ),
+                np.float32
+            )
+        )
+
+        self.assertEqual(
+            get_key(id_score_list_record)(),
+            old_get_sparse_key_logic(id_score_list_record)
+        )
+
+        id_list_record = schema.NewRecord(
+            self.model.net,
+            schema.List(
+                schema.Scalar(
+                    np.int64,
+                    metadata=schema.Metadata(categorical_limit=1000)
+                )
+            )
+        )
+
+        self.assertEqual(
+            get_key(id_list_record)(),
+            old_get_sparse_key_logic(id_list_record)
+        )
