@@ -539,6 +539,28 @@ class TestShapeInference(test_util.TestCase):
         # TODO: find a tighter bound
         assert(np.allclose(x, x_recovered, atol=1e-2))
 
+    def testLearningRateOp(self):
+        net = core.Net("lr_test")
+        iteration = net.ConstantFill(
+            [],
+            "iteration",
+            shape=[1],
+            value=0,
+            dtype=core.DataType.INT64,
+        )
+        lr = net.LearningRate(
+            [iteration],
+            net.NextScopedBlob("weight_decay"),
+            base_lr=0.5,
+            policy="constantWarmup",
+            multiplier=0.0,
+            num_iter=0,
+        )
+        (shapes, types) = workspace.InferShapesAndTypes(
+            [net],
+        )
+        self.assertEqual(shapes['weight_decay'], [1])
+
     def testShapeOp(self):
         model = model_helper.ModelHelper(name="shape_op_test")
         model.Shape('x', 'y')
